/************************** * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
*************************** * * * * * * * * * * * * **************************/

#ifndef _QHACCTABLE_H
#define _QHACCTABLE_H

#include "tableops.h"
#include "resultset.h"

#include <qlist.h>
#include <qdatetime.h>

class QFile;

class ostream;

class QHaccDB;
class QHaccTableIndex;

class QHaccTable : public QHaccResultSet {
 public:
	friend class QHaccTableIndex;

	QHaccTable( const ColType *, const char **, int, int pkcol =-1,
							const char * name =0, uint =5, uint =5, uint maxfree =8 );
	QHaccTable( const QHaccTable& );
	QHaccTable( const QHaccResultSet& );
	QHaccTable& operator=( const QHaccTable& );
	QHaccTable& operator=( const QHaccResultSet& );
	virtual ~QHaccTable();

	bool load( const QHaccTable& );
	bool load( const QHaccResultSet& );
	bool load( QFile&, QString& );
	bool save( QFile&, QString& ) const;
	void startLoad( uint =0 );
	void stopLoad();
	void clear();

	//TableRow at( uint idx ) const;
	//int add( const TableRow& );

	QHaccResultSet toRS() const;

	bool getWhere( TableSelect *, uint size, QHaccResultSet *& results );
	bool getWhere( const TableSelect&, QHaccResultSet *& results );

	bool getWhere( const TableGet&, TableSelect *, uint size, 
								 QHaccResultSet *& results );

	int idcol() const;

	TableCol max( int column );
	TableCol min( int column );
	
	uint deleteWhere( const TableSelect& );
	uint deleteWhere( QHaccResultSet * );

	int updateWhere( const TableSelect&, const TableUpdate& );
	int updateWhere( const TableSelect&, const TableRow& );

	void dropIndex( QHaccTableIndex * );
	void addIndexOn( int col, QHaccTableIndex *& );
	void addIndexOn( int col, int subcol, QHaccTableIndex *& );
	bool contains( const TableCol&, uint& ) const;
	bool contains( const TableRow&, uint& ) const;
	
	void setName( QString );

	static const int DUPLKEY;
	static const int NOTFOUND;

 protected:
	void iresize();
	void iadd();
	virtual int verifyRow( const TableRow * ) const;
	const char * newerror( int code );

	static const int INITINIT, INITGROW, INITFREE;
	static const char * ERRORS[];

 private:
	int pkcol;
	uint maxFree;
	QString name;
	QList<QHaccTableIndex> indexes;
	bool loading;
	QHaccTableIndex * pki;

	int  iadd( TableRow * );
	void iat( uint idx, TableRow *& );
	void update( const TableRow *, const TableRow& );
	void remove( uint idx );
	void deleteWhere( uint * idxarr, uint size );

	void setAutoDelete( bool );
	void igetWhere( const TableSelect&, QHaccTable * rows, QHaccTable *& ret );
	bool getIndexOn( int pos, QHaccTableIndex *& );
	bool getIndexOn( int pos, int spos, QHaccTableIndex *& );
	void removeIndex( QHaccTableIndex * );
	void reindex();
	void remake();

	bool idebug( int, std::ostream *& ) const;
	bool ierror( int, std::ostream *& ) const;
};




class QHaccTableIndex {
 public:
	QHaccTableIndex( const QHaccTableIndex& );
	QHaccTableIndex( QHaccTable *, int );
	QHaccTableIndex( QHaccTable *, int, int );
	QHaccTableIndex& operator=( const QHaccTableIndex& );
	virtual ~QHaccTableIndex();
	
	TableRow at( uint ) const;
	uint rows() const;

	void reindex();
	void remake();

	uint starts( const TableCol& );
	uint ends( const TableCol& );
	bool contains( const TableCol& );
	bool contains( const TableCol&, uint& idx );
	bool contains( const TableRow&, uint& idx );
	void unique( QHaccTable * );
	bool sorts( int ) const;
	bool sorts( int, int ) const;
	int sorts() const;
	uint loc( uint idx ) const;
	TableCol max();
	TableCol min();

 private:
	uint * lookup;
	QHaccTable * data;
	int field, subfield;
	
	void init( QHaccTable *, int, int );
	void at( uint idx, TableRow *& );

	// here are our quicksort functions
	void qsort( uint, uint );
	uint findQ( uint, uint );
	void swap( uint, uint );
	uint ifind( const TableCol&, uint start, uint end, bool& exactFind );
};

#endif
