/************************** * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
*************************** * * * * * * * * * * * * **************************/

#ifndef _QHACCTABLEROWS_H
#define _QHACCTABLEROWS_H

#include "qhaccconstants.h"

#include <qstring.h>
#include <qdatetime.h>

class QHaccTable;
class QHaccTableIndex;

class TableCol {
 public:
	TableCol();
	TableCol( const char * );
	TableCol( const QString& );
	TableCol( int );
	TableCol( uint );
	TableCol( float );
	TableCol( const QDate& );
	TableCol( bool );
	const TableCol& operator=( const TableCol& );

	int compareTo( const TableCol& ) const;

	int geti() const;
	uint getu() const;
	bool getb() const;
	float getf() const;
	QDate getd() const;
	QString gets() const;
	ColType gett() const;

	QString toString() const;
	
 private:
	int idata;
	uint udata;
	float fdata;
	QDate ddata;
	QString sdata;
	bool bdata;
	ColType type;

	void setType( ColType );

	// pointer to the appropriate comparison function
	int ( TableCol::*comparer )( const TableCol& ) const;

	int compareString( const TableCol& ) const;
	int compareBool( const TableCol& ) const;
	int compareInt( const TableCol& ) const;
	int compareUInt( const TableCol& ) const;
	int compareFloat( const TableCol& ) const;
	int compareDate( const TableCol& ) const;
	int compareNull( const TableCol& ) const;
};

class PosVal {
 public:
	PosVal();
	PosVal( int, const TableCol& );
	PosVal( const PosVal& );
	const PosVal& operator=( const PosVal& );
	~PosVal();
	
	int getp() const;
	const TableCol& getv() const;
	void get( int&, TableCol& ) const;

 protected:
	int pos;
	TableCol val;
};

class TableRow {
 public:
	friend class QHaccTable;
	friend class QHaccResultSet;
	friend class QHaccTableIndex;

	TableRow();
	TableRow( const TableRow& );
	TableRow( TableCol *, int );
	virtual ~TableRow();

	QString toString() const;
	const TableRow& operator=( const TableRow& );
	bool isNull() const;

	int geti( int ) const;
	uint getu( int ) const;
	bool getb( int ) const;
	float getf( int ) const;
	QDate getd( int ) const;
	QString gets( int ) const;
	ColType gett( int ) const;
	TableCol get( int ) const;
	bool set( const PosVal& );
	bool set( int, const TableCol& );

	int compareTo( const PosVal& ) const;
	int compareTo( const TableRow&, int ) const;
	int compareTo( const TableRow&, int, int ) const;
	
 protected:
	TableCol * cols;
	int size;

	const TableCol& operator[]( int ) const;
};

#endif



