/************************** * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
*************************** * * * * * * * * * * * * **************************/

#ifndef _RESULTSET_H
#define _RESULTSET_H

#include "qhaccutils.h"
#include "qhacctablerows.h"

#include <qstring.h>

class QHaccDBPlugin;

class QHaccResultSet {
 public:
	friend class QHaccDBPlugin;

	QHaccResultSet( const ColType *, const char ** names, int namesize, 
									uint initsize=5, uint growby=5 );
	QHaccResultSet( const QHaccResultSet& );
	QHaccResultSet& operator=( const QHaccResultSet& );
	virtual ~QHaccResultSet();

	virtual void loadRow( const QString& );
	virtual bool load( const QHaccResultSet& );
	virtual void startLoad( uint =0 );
	virtual void stopLoad();

	virtual TableRow at( uint idx ) const;
	virtual int add( const TableRow& );

	uint rows() const;
	uint columns() const;
	bool isEmpty() const;
	
	QString colname( int ) const;
	ColType coltype( int ) const;

	static const int VALID;
	static const int COLCOUNT;
	static const int COLTYPE;
	static const int NULLROW;
	static const int LOWERROR;
	const char * error( int code ) const;

 protected:
	virtual void iadd();
	virtual void iresize();
	virtual int verifyRow( const TableRow * ) const;
	virtual const char * newerror( int code ) const;
	static const char * ERRORS[];

	int cols;
	bool deleter;
	uint size, arrSize, growBy; // how many rows? how big an array? growth?
	QString * names;
	ColType * types;
	TableRow ** data;

	void resizeTo( uint );
};

#endif
