/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************** * * * * * * * * * * * * **************************/

#ifndef SQLDBPLUGINS_H
#define SQLDBPLUGINS_H

#include "plugins.h"

/****************************************************************/
/** QHACCSQLDBPLUGIN                                           **/
/**   An abstract class to interact with a sql database        **/
/**                                                            **/
/****************************************************************/
class QHaccSQLDBPlugin : public QHaccDBPlugin{
 public:
	QHaccSQLDBPlugin();
	virtual ~QHaccSQLDBPlugin();

	virtual bool save( const QString& home, bool protect, QString& error ) =0;
	bool load( Table, QHaccResultSet& );
	virtual bool load( QHacc * engine, const QString& home, QString& err ) =0;
	virtual QString description() const =0;
	virtual bool imprt( QHaccResultSet * );
	virtual bool exprt( QHaccResultSet * );

	/**
	 * Perform a select statement on the database
	 * @param stmt The query to run
	 * @param ret The data returned
	 * @return True, if returned resultset is non-empty
	 **/
	virtual bool sel( const QString& stmt, QHaccResultSet& ret ) =0;

	/**
	 * Perform an update, delete on the database
	 * @param stmt The SQL to run
	 * @param the number of rows affected, or one that cannot be determined
	 **/
	virtual int  run( const QString& stmt ) =0;
	virtual QString imin( Table, int col ) =0;
	virtual QString imax( Table, int col ) =0;

	/**
	 * Gets a fieldname for a SQL statemtn in a QHacc-friendly format
	 * @param fieldname The name of the field to be selected
	 * @param ct The field's datatype
	 * @return Valid SQL for returning the given field in QHacc-accessable format
	 **/
	virtual QString selField( const QString& fieldname, ColType ct ) const =0;

	/**
	 * @returns A field identifier suitable for database insertion
	 **/
	virtual QString sqlField( const TableCol& ) const =0;

	/**
	 * @returns a field identifier suitable for where-clauses
	 **/
	virtual QString sqlField( const TableSelect& ) const =0;

	/**
	 * @returns a selection string suitable for select-clauses
	 **/
	//virtual QString sqlField( const TableGet& ) const =0;

	virtual bool dirty() const;
	virtual void startLoad( Table t, uint =0 );
	virtual void stopLoad( Table t );


	// these functions satisfy the QHaccDBPlugin interface, but the
	// above functions will do all the work
	int add( Table, const TableRow& );

	int updateWhere( Table, const TableSelect&, const TableUpdate& );
	int updateWhere( Table, const TableSelect&, const TableRow& );

	bool getWhere( Table, TableSelect *, uint size,
												 QHaccResultSet *& results );
	bool getWhere( Table, const TableGet&, TableSelect *, uint size,
								 QHaccResultSet *& results );
	bool getWhere( Table, const TableSelect&, QHaccResultSet *& results );

	uint deleteWhere( Table, const TableSelect& );
	uint deleteWhere( Table, QHaccResultSet * );

	TableCol max( Table, int col );
	TableCol min( Table, int col );

 protected:
	/**
	 * Convert a QHacc tablerow into a string suitable for 
	 * insertion to the given table. This function only provides the 
	 * part of the insert statement after 'values'
	 * @param t The table to get the insertion
	 * @param r The row to be inserted
	 * @return The valid
	 **/
	QString iconv( Table t, const TableRow& r ) const;

	/**
	 * String to in table lookup
	 * @return Table id
	 **/
	static int table( const QString& );

	/**
	 * Table to tablename lookup
	 * @return tablename
	 **/
	static QString table( Table t );

	/**
	 * Convert the given string and column type into a QHacc-friendly tablecol.
	 * @param str The string returned from the database
	 * @param ct The desired ColType
	 * @return A QHacc-friendly data structure
	 **/
	TableCol maketc( const QString& str, ColType ct ) const;
};

#endif
